# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import os
import re
from .version import Version
from .logging import debugLogger as logger
from subprocess import check_output

"""
Handles differences between different distributions
"""
__author__ = "Dave Malcolm <dmalcolm@redhat.com>, Zack Cerza <zcerza@redhat.com>"


class DistributionNotSupportedError(Exception):  # pragma: no cover
    """
    This distribution is not supported.
    """
    PATCH_MESSAGE = "Please send patches to dogtail-devel-list@gnome.org"

    def __init__(self, distro):
        self.distro = distro

    def __str__(self):
        return self.distro + ". " + DistributionNotSupportedError.PATCH_MESSAGE


class PackageNotFoundError(Exception):
    """
    Error finding the requested package.
    """
    pass

global packageDb
global distro


class PackageDb(object):
    """
    Class to abstract the details of whatever software package database is in
    use (RPM, APT, etc)
    """

    def __init__(self):
        self.prefix = '/usr'
        self.localePrefixes = [self.prefix + '/share/locale']

    def getVersion(self, packageName):
        """
        Method to get the version of an installed package as a Version
        instance (or raise an exception if not found)

        Note: does not know about distributions' internal revision numbers.
        """
        raise NotImplementedError

    def getFiles(self, packageName):
        """
        Method to get a list of filenames owned by the package, or raise an
        exception if not found.
        """
        raise NotImplementedError

    def getMoFiles(self, locale=None):
        """
        Method to get a list of all .mo files on the system, optionally for a
        specific locale.
        """
        moFiles = {}

        def appendIfMoFile(moFiles, dirName, fNames):
            import re
            for fName in fNames:
                if re.match('(.*)\\.mo', fName):
                    moFiles[dirName + '/' + fName] = None

        for localePrefix in self.localePrefixes:
            if locale:
                localePrefix = localePrefix + '/' + locale
            os.walk(localePrefix, appendIfMoFile, moFiles)

        return list(moFiles.keys())

    def getDependencies(self, packageName):
        """
        Method to get a list of unique package names that this package
        is dependent on, or raise an exception if the package is not
        found.
        """
        raise NotImplementedError


class _RpmPackageDb(PackageDb):  # pragma: no cover
    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        import rpm
        ts = rpm.TransactionSet()
        for header in ts.dbMatch("name", packageName):
            return Version.fromString(header["version"])
        raise PackageNotFoundError(packageName)

    def getFiles(self, packageName):
        import rpm
        ts = rpm.TransactionSet()
        for header in ts.dbMatch("name", packageName):
            return header["filenames"]
        raise PackageNotFoundError(packageName)

    def getDependencies(self, packageName):
        import rpm
        ts = rpm.TransactionSet()
        for header in ts.dbMatch("name", packageName):
            # Simulate a set using a hash (to a dummy value);
            # sets were only added in Python 2.4
            result = {}

            # Get the list of requirements; these are
            # sometimes package names, but can also be
            # so-names of libraries, and invented virtual
            # ids
            for requirement in header[rpm.RPMTAG_REQUIRES]:
                # Get the name of the package providing
                # this requirement:
                for depPackageHeader in ts.dbMatch("provides", requirement):
                    depName = depPackageHeader['name']
                    if depName != packageName:
                        # Add to the Hash with a dummy value
                        result[depName] = None
            return list(result.keys())
        raise PackageNotFoundError(packageName)


class _AptPackageDb(PackageDb):
    def __init__(self):
        PackageDb.__init__(self)
        self.cache = None

    def getVersion(self, packageName):
        if not self.cache:
            import apt_pkg
            apt_pkg.init()
            self.cache = apt_pkg.Cache()
        packages = self.cache.packages
        for package in packages:
            if package.name == packageName:
                verString = re.match('.*Ver:\'(.*)-.*\' Section:', str(package.current_ver)).group(1)
                return Version.fromString(verString)
        raise PackageNotFoundError(packageName)

    def getFiles(self, packageName):
        files = []
        list = os.popen('dpkg -L %s' % packageName).readlines()
        if not list:
            raise PackageNotFoundError(packageName)
        else:
            for line in list:
                file = line.strip()
                if file:
                    files.append(file)
            return files

    def getDependencies(self, packageName):
        # Simulate a set using a hash (to a dummy value);
        # sets were only added in Python 2.4
        result = {}
        if not self.cache:
            import apt_pkg
            apt_pkg.init()
            self.cache = apt_pkg.Cache()
        packages = self.cache.packages
        for package in packages:
            if package.name == packageName:
                current = package.current_ver
                if not current:
                    raise PackageNotFoundError(packageName)
                depends = current.depends_list
                list = depends['Depends']
                for dependency in list:
                    name = dependency[0].target_pkg.name
                    # Add to the hash using a dummy value
                    result[name] = None
        return list(result.keys())


class _UbuntuAptPackageDb(_AptPackageDb):
    def __init__(self):
        _AptPackageDb.__init__(self)
        self.localePrefixes.append(self.prefix + '/share/locale-langpack')


class _PortagePackageDb(PackageDb):  # pragma: no cover
    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        # the portage utilities are almost always going to be in
        # /usr/lib/portage/pym
        import sys
        sys.path.append('/usr/lib/portage/pym')
        import portage
        # FIXME: this takes the first package returned in the list, in the
        # case that there are slotted packages, and removes the leading
        # category such as 'sys-apps'
        gentooPackageName = portage.db["/"]["vartree"].dbapi.match(packageName)[0].split('/')[1]
        # this removes the distribution specific versioning returning only the
        # upstream version
        upstreamVersion = portage.pkgsplit(gentooPackageName)[1]
        # print("Version of package is: " + upstreamVersion)
        return Version.fromString(upstreamVersion)


class _ConaryPackageDb(PackageDb):  # pragma: no cover
    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        from conaryclient import ConaryClient
        client = ConaryClient()
        dbVersions = client.db.getTroveVersionList(packageName)
        if not len(dbVersions):
            raise PackageNotFoundError(packageName)
        return dbVersions[0].trailingRevision().asString().split("-")[0]

# getVersion not implemented because on Solaris multiple modules are installed
# in single packages, so it is hard to tell what version number of a specific
# module.


class _SolarisPackageDb(PackageDb):  # pragma: no cover
    def __init__(self):
        PackageDb.__init__(self)


class JhBuildPackageDb(PackageDb):  # pragma: no cover
    def __init__(self):
        PackageDb.__init__(self)
        prefixes = []
        prefixes.append(os.environ['LD_LIBRARY_PATH'])
        prefixes.append(os.environ['XDG_CONFIG_DIRS'])
        prefixes.append(os.environ['PKG_CONFIG_PATH'])
        self.prefix = os.path.commonprefix(prefixes)
        self.localePrefixes.append(self.prefix + '/share/locale')

    def getDependencies(self, packageName):
        result = {}
        lines = os.popen('jhbuild list ' + packageName).readlines()
        for line in lines:
            if line:
                result[line.strip()] = None
        return list(result.keys())


class _ContinuousPackageDb(PackageDb):

    def __init__(self):
        PackageDb.__init__(self)

    def getVersion(self, packageName):
        return ''

    def getFiles(self, packageName):
        return check_output(
            ["ls -1 /usr/share/locale/*/LC_MESSAGES/%s.mo" % packageName], shell=True).strip().split('\n')

    def getDependencies(self, packageName):
        # Simulate a set using a hash (to a dummy value);
        # sets were only added in Python 2.4
        return []


class Distro(object):
    """
    Class representing a distribution.

    Scripts may want to do arbitrary logic based on whichever distro is in use
    (e.g. handling differences in names of packages, distribution-specific
    patches, etc.)

    We can either create methods in the Distro class to handle these, or we
    can use constructs like isinstance(distro, Ubuntu) to handle this. We can
    even create hierarchies of distro subclasses to handle this kind of thing
    (could get messy fast though)
    """


class Fedora(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _RpmPackageDb()


class RHEL(Fedora):  # pragma: no cover
    pass


class Debian(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _AptPackageDb()


class Ubuntu(Debian):
    def __init__(self):
        self.packageDb = _UbuntuAptPackageDb()


class Suse(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _RpmPackageDb()


class Gentoo(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _PortagePackageDb()


class Conary(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _ConaryPackageDb()


class Solaris(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _SolarisPackageDb()


class JHBuild(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = JhBuildPackageDb()


class GnomeContinuous(Distro):  # pragma: no cover
    def __init__(self):
        self.packageDb = _ContinuousPackageDb()


def detectDistro():
    logger.log("Detecting distribution:", newline=False)

    if os.environ.get("CERTIFIED_GNOMIE", "no") == "yes":
        distro = JHBuild()  # pragma: no cover
    elif os.path.exists("/etc/SuSE-release"):
        distro = Suse()  # pragma: no cover
    elif os.path.exists("/etc/fedora-release"):
        distro = Fedora()  # pragma: no cover
    elif os.path.exists("/etc/redhat-release"):
        distro = RHEL()  # pragma: no cover
    elif os.path.exists("/usr/share/doc/ubuntu-minimal"):
        distro = Ubuntu()
    elif os.path.exists("/etc/debian_version"):  # pragma: no cover
        distro = Debian()  # pragma: no cover
    elif os.path.exists("/etc/gentoo-release"):  # pragma: no cover
        distro = Gentoo()  # pragma: no cover
    elif os.path.exists("/etc/slackware-version"):  # pragma: no cover
        raise DistributionNotSupportedError("Slackware")  # pragma: no cover
    elif os.path.exists("/var/lib/conarydb/conarydb"):  # pragma: no cover
        distro = Conary()  # pragma: no cover
    elif os.path.exists("/etc/release") and \
            re.match(".*Solaris", open("/etc/release").readline()):  # pragma: no cover
        distro = Solaris()  # pragma: no cover
    elif os.path.exists("/etc/os-release") and \
            re.match(".*GNOME-Continuous", open("/etc/os-release").readline()):  # pragma: no cover
        distro = GnomeContinuous()  # pragma: no cover
    else:
        raise DistributionNotSupportedError("Unknown")  # pragma: no cover
    logger.log(distro.__class__.__name__)
    return distro

distro = detectDistro()
packageDb = distro.packageDb



#!/usr/bin/env python3 # pylint: disable=too-many-lines
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
import os
import re
from configparser import ConfigParser
from configparser import NoOptionError

from src import logger


def get_desktop_file_location_by_config(name):
    """
     根据桌面配置文件获取文件或者文件夹坐标
    :param name: 文件名称
    :return: 坐标
    """
    # 不同图标大小的高宽 y，x
    num = [(102, 62), (118, 92), (148, 120), (172, 174), (207, 242)]
    desktop_conf_file = os.path.expanduser("~/.config/deepin/dde-desktop/dde-desktop.conf")
    conf = ConfigParser()
    if not os.path.exists(desktop_conf_file):
        os.system("cd ~/Desktop;touch .test_desktop;sleep 1;rm -rf .test_desktop")
    try:
        conf.read(os.path.expanduser(desktop_conf_file))
        level = int(conf.get("GeneralConfig", "IconLevel", fallback=1))
    except NoOptionError as exc:
        raise EnvironmentError(f"{desktop_conf_file} 可能不存在") from exc
    for _x, _y in dict(conf["SingleScreen"]).items():
        frame = re.sub(
            r"\\x[0-9a-fA-F]{2,4}",
            lambda i: chr(int(i.group().replace(r"\x", ""), 16)),
            _y,
        )
        if frame.endswith(name):
            axes = _x.split("_")[::-1]
            _y, _x = map(
                lambda i: int(i[0]) * int(i[1]) + int(i[1]) / 2,
                zip(axes, num[level]),
            )
            logger.info(f"{name}坐标为{str(_x)},{str(_x)}")
            return int(_x), int(_y)
    raise ValueError(f"{name} 未找到")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
from contextlib import contextmanager
from typing import Union

from setting.globalconfig import GlobalConfig

try:
    from playwright.sync_api import sync_playwright
    from playwright.sync_api import Page
    from playwright.sync_api import LocatorAssertions
    from playwright.sync_api import expect as _expect
    from playwright.sync_api import Locator
    from playwright.sync_api import APIResponse
    from playwright.sync_api import PageAssertions
    from playwright.sync_api import APIResponseAssertions
except ImportError:
    print("Please install playwright")


class WebUI:
    def __init__(self, page: Page):
        self.page = page

    def goto(self, url):
        self.page.goto(url)

    def input_text(self, element, text):
        self.page.fill(element, text)

    def click_element(self, element):
        self.page.click(element)


class WebAssert:
    @staticmethod
    def assert_locator(
        locator: Union[Page, Locator, APIResponse],
    ) -> Union[PageAssertions, LocatorAssertions, APIResponseAssertions]:
        return _expect(locator)


@contextmanager
def debug_page():
    driver = sync_playwright().start()
    browser = driver.chromium.launch_persistent_context(
        user_data_dir=GlobalConfig.USER_DATE_DIR,
        executable_path=GlobalConfig.EXECUTABLE_PATH,
        ignore_https_errors=True,
        no_viewport=True,
        slow_mo=500,
        headless=False,
        bypass_csp=True,
        args=[
            "--disable-blink-features=AutomationControlled",
            "--start-maximized",
        ],
    )
    _page = browser.pages[0]
    yield _page
    browser.close()
    driver.stop()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
import json
import os

# from concurrent.futures import ALL_COMPLETED
# from concurrent.futures import ThreadPoolExecutor
# from concurrent.futures import wait
from time import sleep
from urllib.parse import urlencode

from setting import conf
from setting.globalconfig import FixedCsvTitle
from src.pms._base import _Base
from src.rtk._base import transform_app_name

__all__ = []


class Csv2Pms(_Base):
    __author__ = "mikigo<huangmingqiang@uniontech.com>"

    config_error_log = "请检查您传递的 '命令行参数' 或 setting/globalconfig.ini 里的配置项"

    def __init__(self, app_name: str = None, user: str = None, password: str = None, csv_name=None):
        super().__init__(user=user, password=password)
        self.username = user
        self.password = user
        self.base_url = "https://pms.uniontech.com"
        self.walk_dir = (
            f"{conf.APPS_PATH}/{transform_app_name(app_name)}" if app_name else conf.APPS_PATH
        )
        self.csv_name = csv_name or conf.CSV_NAME_TO_PMS

        if not self.csv_name:
            raise EnvironmentError(self.config_error_log)

    def get_csv_info(self):
        csv_path = None
        for root, _, files in os.walk(self.walk_dir):
            for file in files:
                if f"{self.csv_name}.csv" == file:
                    csv_path = f"{root}/{file}"
        if csv_path is None:
            raise FileNotFoundError(self.config_error_log)

        with open(csv_path, "r", encoding="utf-8") as f:
            txt_list = f.readlines()

        csv_heads = txt_list[0].strip().split(",")
        csv_head_index_map = {}
        for index, title in enumerate(csv_heads):
            for i in FixedCsvTitle:
                if i.value == title.strip():
                    csv_head_index_map[i.name] = {
                        "head_name": i.value,
                        "head_index": index,
                    }

        taglines = [txt.strip().split(",") for txt in txt_list[1:]]
        return csv_head_index_map, taglines

    def post_to_pms(self):
        csv_head_index_map, taglines = self.get_csv_info()

        def csv_map(x):
            return csv_head_index_map.get(x).get("head_index")

        def push(value, index):
            sleep(1)
            case_id = value[csv_map(FixedCsvTitle.pms_case_id.name)]
            case_url = f"{self.base_url}/testcase-view-{case_id}.json"
            case_res = json.loads(self.rx.open_url(case_url, timeout=10))
            case_data = json.loads(case_res.get("data"))
            case_title = case_data.get("title")
            rel_case_title = (
                case_title.split(case_id)[-1]
                .rstrip(f"{case_data.get('libName')}")
                .strip(" - ")
                .strip(" ")
            )

            edit_url = f"{self.base_url}/testcase-edit-{case_id}.html"
            data = {
                "title": rel_case_title,
                "isAutomation": "是",
            }
            try:
                if value[csv_map(FixedCsvTitle.device_type.name)] == "PPL":
                    data["deviceType"] = "PPL(外设)"
            except AttributeError:
                pass
            try:
                if value[csv_map(FixedCsvTitle.device_type.name)] == "COL":
                    data["deviceType"] = "COL(主控)"
            except AttributeError:
                pass
            try:
                if value[csv_map(FixedCsvTitle.case_from.name)] == "是":
                    data["caseSource"] = "是"
            except AttributeError:
                pass
            try:
                if value[csv_map(FixedCsvTitle.online_obj.name)] == "CICD":
                    data["lineCD"] = "是"
            except AttributeError:
                pass
            bytes_data = urlencode(data).encode("utf-8")
            res = self.rx.session.open(fullurl=edit_url, data=bytes_data, timeout=10)
            print(f"({index}) {case_id} {data} {res.status}")

        # tasks = []
        # executor = ThreadPoolExecutor()
        for index, value in enumerate(taglines):
            push(value, index)
            # t = executor.submit(push, value, index)
            # tasks.append(t)
        # wait(tasks, return_when=ALL_COMPLETED)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from .logging import debugLogger as logger
import inspect

"""
General exceptions; not overly module-specific
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"


def warn(message, caller=True):
    """
    Generate a warning, and pass it to the debug logger.
    """
    frameRec = inspect.stack()[-1]
    message = "Warning: %s:%s: %s" % (frameRec[1], frameRec[2], message)
    if caller and frameRec[1] != '<stdin>' and frameRec[1] != '<string>':
        message = message + ':\n  ' + frameRec[4][0]
    del frameRec
    logger.log(message)


class DependencyNotFoundError(Exception):
    """
    A dependency was not found.
    """
    pass



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=E0401,C0413,R0903,W0707,W0611
import random

from pdocr_rpc import OCR as _OCR
from pdocr_rpc.conf import setting as ocr_setting

from setting import conf


class OCRUtils:
    ocr_setting.NETWORK_RETRY = int(conf.OCR_NETWORK_RETRY)
    ocr_setting.PAUSE = float(conf.OCR_PAUSE)
    ocr_setting.TIMEOUT = float(conf.OCR_TIMEOUT)
    ocr_setting.MAX_MATCH_NUMBER = int(conf.OCR_MAX_MATCH_NUMBER)
    ocr_setting.PORT = conf.OCR_PORT
    ocr_servers = [i.strip() for i in conf.OCR_SERVER_HOST.split("/") if i]
    x = None
    y = None
    result = None

    @classmethod
    def ocr(cls, *args, **kwargs):
        """ocr load balance"""
        servers = cls.ocr_servers
        while servers:
            ocr_setting.SERVER_IP = random.choice(servers)
            if _OCR.check_connected() is False:
                servers.remove(ocr_setting.SERVER_IP)
                ocr_setting.SERVER_IP = None
            else:
                break
        if ocr_setting.SERVER_IP is None:
            raise EnvironmentError(f"所有OCR服务器不可用: {cls.ocr_servers}")
        result = _OCR.ocr(*args, **kwargs)
        return result

    @classmethod
    def ocr_remote(
        cls,
        target_strings: tuple = None,
        picture_abspath: str = None,
        similarity: [int, float] = 0.6,
        return_default: bool = False,
        return_first: bool = False,
        lang: str = "ch",
        network_retry: int = None,
        pause: [int, float] = None,
        timeout: [int, float] = None,
        max_match_number: int = None,
    ):
        servers = cls.ocr_servers
        while servers:
            ocr_setting.SERVER_IP = random.choice(servers)
            if _OCR.check_connected() is False:
                servers.remove(ocr_setting.SERVER_IP)
                ocr_setting.SERVER_IP = None
            else:
                break
        if ocr_setting.SERVER_IP is None:
            raise EnvironmentError(f"所有OCR服务器不可用: {cls.ocr_servers}")
        return _OCR.ocr(
            *target_strings,
            picture_abspath=picture_abspath,
            similarity=similarity,
            return_default=return_default,
            return_first=return_first,
            lang=lang,
            network_retry=network_retry,
            pause=pause,
            timeout=timeout,
            max_match_number=max_match_number,
        )

    @classmethod
    def ocrx(cls, *args, **kwargs):
        """
        支持链式调用
        ocrx().click()
        """
        cls.result = cls.ocr(*args, **kwargs)
        if isinstance(cls.result, tuple):
            cls.x, cls.y = cls.result
            return cls
        return cls.result

    @classmethod
    def _check_xy(cls):
        if cls.x is None and cls.y is None:
            raise ValueError("ocrx 没有识别到")

    @classmethod
    def click(cls):
        from src.mouse_key import MouseKey

        cls._check_xy()
        MouseKey.click(cls.x, cls.y)
        return cls

    @classmethod
    def right_click(cls):
        from src.mouse_key import MouseKey

        cls._check_xy()
        MouseKey.right_click(cls.x, cls.y)
        return cls

    @classmethod
    def double_click(cls):
        from src.mouse_key import MouseKey

        cls._check_xy()
        MouseKey.double_click(cls.x, cls.y)
        return cls

    @classmethod
    def center(cls):
        cls._check_xy()
        return cls.x, cls.y

    @classmethod
    def all_result(cls):
        return cls.result

    @classmethod
    def ocr_find_by_range(cls, text, x1=None, x2=None, y1=None, y2=None):
        """
        OCR在当前界面中识别到多个关键词时，通过区域筛选出对应关键词并返回坐标
        :param text: 页面范围内查找关键词，可自由使用以下参数划定查找区域
        :param x1: x坐标开始范围，有效区域为大于 x1 区域
        :param x2: x坐标结束范围，有效区域为小于 x1 区域
        :param y1: y坐标开始范围，有效区域为大于 y1 区域
        :param y2: y坐标结束范围，有效区域为小于 y2 区域
        :return: 坐标元组 (x, y)

        注意：该方法设计是为了筛选出唯一坐标，所以需要特定区域内只有一组OCR关键词，若任有多组数据会直接报错，请增加精度

        以默认分辨率 1920*1080 为例，多种示例情况如下：
        示例1（识别左半屏幕关键字）：ocr_find_by_range(x2=960)
        示例2（识别下半屏幕关键字）：ocr_find_by_range(y1=540)
        示例3（识别右下半屏幕关键字）：ocr_find_by_range(x1=960, y1=540)
        示例4（识别特定区域 ：100*900-200*950 内关键字）：ocr_find_by_range(x1=100, y1=900, x2=200, y2=950)
        """
        defaults = {"x1": 0, "x2": 1920, "y1": 0, "y2": 1080}

        x1 = x1 if x1 is not None else defaults["x1"]
        x2 = x2 if x2 is not None else defaults["x2"]
        y1 = y1 if y1 is not None else defaults["y1"]
        y2 = y2 if y2 is not None else defaults["y2"]

        if x1 > x2 or y1 > y2:
            raise ValueError("x1 > x2 or y1 > y2")

        results = []
        ocr_return = cls.ocr(text)
        if isinstance(ocr_return, dict):
            for _, value in ocr_return.items():
                if x1 <= value[0] <= x2 and y1 <= value[1] <= y2:
                    results.append(value)
            if len(results) == 0:
                raise ValueError(
                    f"范围内[{x1, y1} - {x2, y2}]未识别到关键词[{text}]，请增加识别精度，识别结果为：{results}"
                )
            elif len(results) == 1:
                return results[0]
            else:
                raise ValueError(
                    f"范围内[{x1, y1} - {x2, y2}]识别到多组关键词[{text}]，请增加识别精度，识别结果为：{results}"
                )
        if x1 <= ocr_return[0] <= x2 and y1 <= ocr_return[1] <= y2:
            return ocr_return
        else:
            raise ValueError(
                f"范围内[{x1, y1} - {x2, y2}]识别不到关键词，请增加识别精度，识别结果为：{ocr_return}"
            )



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import re
import locale
from datetime import datetime
from datetime import timedelta
from collections import deque

from setting import conf
from src.rtk._base import transform_app_name

locale.setlocale(locale.LC_ALL, "en_US.UTF-8")


class Commit:
    __author__ = "mikigo<huangmingqiang@uniontech.com>"

    def __init__(self, app_name: str, branch: str, startdate: str, enddate: str = None):
        self.app_name = transform_app_name(app_name)
        self.startdate = datetime.strptime(startdate, "%Y-%m-%d")
        self.enddate = (
            self.now_dt
            if enddate == "" or enddate is None
            else datetime.strptime(enddate, "%Y-%m-%d") + timedelta(days=1)
        )
        self.branch = branch
        if self.branch is None:
            raise ValueError("branch 参数必传")

    @property
    def now_dt(self):
        return datetime.strptime(datetime.now().strftime("%Y-%m-%d-%H-%M"), "%Y-%m-%d-%H-%M")

    @property
    def git_logs(self) -> list:
        _git_logs = (
            os.popen(f"cd {conf.APPS_PATH}/{self.app_name} && git log {self.branch}")
            .read()
            .splitlines()
        )
        git_logs = []
        tmp = []
        for line in _git_logs:
            if line.startswith("commit "):
                if tmp:
                    git_logs.append(tmp)
                    tmp = []
                tmp.append(line.split(" ")[1].strip())
            elif line.startswith("Author:"):
                tmp.append(line.split("Author: ")[-1])
            elif line.startswith("Date: "):
                tmp.append(line.split("Date: ")[1].strip())
            else:
                if line.strip() and not line.strip().startswith("Change-Id"):
                    tmp.append(line.strip())
        return git_logs

    def commit_id(self):
        commit_ids = deque()
        for commit_id, author, _time_str, *msg in self.git_logs:
            time_str = " ".join(_time_str.split(" ")[:-1])
            try:
                git_dt = datetime.strptime(time_str, "%a %b %d %H:%M:%S %Y")
            except ValueError:
                continue

            if self.startdate <= git_dt <= self.enddate:
                commit_ids.appendleft([commit_id, author, git_dt, msg])

        if not commit_ids:
            raise ValueError(f"{self.startdate} 到 {self.enddate} 没有获取到有效的 commit id")

        return commit_ids


if __name__ == "__main__":
    Commit(
        app_name="apps/autotest_kernel",
        branch="at-develop/eagle",
        startdate="2024-05-20",
    ).commit_id()



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
from setting import conf


def check_git_installed():
    if not os.popen("git --version").read().startswith("git version"):
        print("git 没有安装，我们将尝试安装")
        os.system(f"echo '{conf.PASSWORD}' | sudo -S apt install git -y")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
from typing import Union, List

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
# pylint: disable=E0401,C0413,R0903,W0707,W0611
try:
    import cv2 as cv

    GET_OPENCV_FORM_RPC = False
except ModuleNotFoundError:
    GET_OPENCV_FORM_RPC = True

from image_center import ImageCenter
from image_center.conf import setting as image_setting

from setting import conf

image_setting.PORT = conf.OPENCV_PORT
image_setting.SERVER_IP = conf.OPENCV_SERVER_HOST
image_setting.NETWORK_RETRY = int(conf.OPENCV_NETWORK_RETRY)
image_setting.PAUSE = float(conf.OPENCV_PAUSE)
image_setting.TIMEOUT = float(conf.OPENCV_TIMEOUT)
image_setting.MAX_MATCH_NUMBER = int(conf.OPENCV_MAX_MATCH_NUMBER)
image_setting.IMAGE_RATE = float(conf.IMAGE_RATE)


class ImageUtils(ImageCenter):
    """图像识别的工具类"""

    @classmethod
    def find_image_remote(
            cls,
            widget: tuple,
            rate: Union[float, int] = None,
            multiple: bool = False,
            picture_abspath: str = None,
            screen_bbox: List[int] = None,
            log_level: str = "info",
            network_retry: int = None,
            pause: [int, float] = None,
            timeout: [int, float] = None,
            max_match_number: int = None,
    ):
        return cls.find_image(
            *widget,
            rate=rate,
            multiple=multiple,
            picture_abspath=picture_abspath,
            screen_bbox=screen_bbox,
            log_level=log_level,
            network_retry=network_retry,
            pause=pause,
            timeout=timeout,
            max_match_number=max_match_number,
        )



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114,C0115,C0116,R1722,W1514,C0103,W1514,C0103,R1721,R0912,W0612
import os
import re
import shutil
from time import strftime
from configparser import ConfigParser

from setting.globalconfig import GlobalConfig
from setting.globalconfig import FixedCsvTitle


class StartApp:
    def __init__(self, app_name: str):
        self.app_name = app_name.strip("/")

    def copy_template_to_apps(self):
        """copy_template_to_apps"""
        app_name_path = f"{GlobalConfig.APPS_PATH}/{self.app_name}"
        if os.path.exists(app_name_path) and os.listdir(app_name_path):
            if input(
                f"{GlobalConfig.APPS_PATH}/{self.app_name}目录存在且里面存在文件，请确认是否清空（Y/N）："
            ) in ("y", "Y"):
                os.system(f"rm -rf {app_name_path}/*")
            else:
                exit(0)
        if not os.path.exists(app_name_path):
            os.makedirs(app_name_path)
        os.system(
            "cp -r " f"{GlobalConfig.SETTING_PATH}/template/app_template/* " f"{app_name_path}/"
        )
        os.system(
            "cp -r "
            f"{GlobalConfig.SETTING_PATH}/template/app_template/.gitignore-tpl "
            f"{app_name_path}/"
        )

    def rewrite(self):
        """rewrite"""
        for root, dirs, files in os.walk(f"{GlobalConfig.APPS_PATH}/{self.app_name}"):
            for file in files:
                app_name = self.app_name
                if self.app_name.startswith("autotest_"):
                    app_name = re.sub(r"autotest_", "", self.app_name)

                if file.endswith("-tpl"):
                    shutil.move(f"{root}/{file}", f"{root}/{file[:-4]}")
                    file = file[:-4]

                if "${app_name}" in file:
                    new_file = re.sub(
                        r"\${app_name}", app_name if app_name else self.app_name, file
                    )
                    shutil.move(f"{root}/{file}", f"{root}/{new_file}")
                    file = new_file

                if ".py" in file or ".csv" in file:
                    with open(f"{root}/{file}", "r") as f:
                        codes = f.readlines()
                    new_codes = []
                    for code in codes:
                        if "##" in code:
                            code = code.replace("##", "")
                        if "${APP_NAME}" in code:
                            code = re.sub(r"\${APP_NAME}", self.app_name, code)
                        if "${app_name}" in code:
                            code = re.sub(r"\${app_name}", app_name, code)
                        if "${APP-NAME}" in code:
                            code = re.sub(r"\${APP-NAME}", app_name.replace("_", "-"), code)
                        if "${AppName}" in code:
                            code = re.sub(r"\${AppName}", app_name.title().replace("_", ""), code)
                        if "${USER}" in code:
                            code = re.sub(r"\${USER}", GlobalConfig.USERNAME, code)
                        if "${DATE}" in code:
                            code = re.sub(r"\${DATE}", strftime("%Y/%m/%d"), code)
                        if "${TIME}" in code:
                            code = re.sub(r"\${TIME}", strftime("%H:%M:%S"), code)
                        if "${FIXEDCSVTITLE}" in code:
                            code = re.sub(
                                r"\${FIXEDCSVTITLE}",
                                ",".join([i.value for i in FixedCsvTitle]),
                                code,
                            )
                        new_codes.append(code)
                    with open(f"{root}/{file}", "w") as f:
                        f.writelines([i for i in new_codes])

                if file == "control":
                    conf = ConfigParser()
                    conf.read(f"{root}/{file}")
                    conf.set("Depends", "autotest-basic-frame", GlobalConfig.current_tag)
                    with open(f"{root}/{file}", "w", encoding="utf-8") as f:
                        conf.write(f)

        print(f"{self.app_name} 创建成功,请查看 {GlobalConfig.APPS_PATH}/{self.app_name}")



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-License-Identifier: GPL-2.0-only
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
import json
import os
from collections import Counter
from enum import Enum
from enum import unique
from os.path import exists

from setting import conf
from setting.globalconfig import GlobalConfig
from src import sleep


@unique
class Args(Enum):
    app_name = "app_name"
    keywords = "keywords"
    tags = "tags"
    reruns = "reruns"
    record_failed_case = "record_failed_case"
    clean = "clean"
    report_formats = "report_formats"
    max_fail = "max_fail"
    log_level = "log_level"
    timeout = "timeout"
    debug = "debug"
    noskip = "noskip"
    ifixed = "ifixed"
    send_pms = "send_pms"
    task_id = "task_id"
    trigger = "trigger"
    resolution = "resolution"
    case_file = "case_file"
    branch = "branch"
    deb_path = "deb_path"
    pms_user = "pms_user"
    pms_password = "pms_password"
    suite_id = "suite_id"
    pms_info_file = "pms_info_file"
    top = "top"
    lastfailed = "lastfailed"
    duringfail = "duringfail"
    repeat = "repeat"
    project_name = "project_name"
    build_location = "build_location"
    line = "line"
    clients = "clients"
    send_code = "send_code"
    build_env = "build_env"
    client_password = "client_password"
    parallel = "parallel"
    autostart = "autostart"
    pyid2csv = "pyid2csv"
    export_csv_file = "export_csv_file"
    pms2csv = "pms2csv"
    pms_link_csv = "pms_link_csv"
    send2task = "send2task"
    startdate = "startdate"
    enddate = "enddate"
    git_url = "git_url"
    git_user = "git_user"
    git_password = "git_password"
    depth = "depth"
    path_to = "path_to"
    execution_mode = "execution_mode"
    slaves = "slaves"
    pms_case_file_path = "pms_case_file_path"
    json_backfill_base_url = "json_backfill_base_url"
    json_backfill_task_id = "json_backfill_task_id"
    json_backfill_user = "json_backfill_user"
    json_backfill_password = "json_backfill_password"
    json_backfill_custom_api = "json_backfill_custom_api"


def transform_app_name(app_name):
    if not app_name:
        return None
    if "-" in app_name:
        raise ValueError(f"{app_name} 中存在 ['-'] 符号")
    if app_name.startswith("apps/"):
        app_name = app_name.replace("apps/", "").strip("/")
    for dir_name in os.listdir(conf.APPS_PATH):
        if dir_name == app_name:
            return app_name
    raise NotADirectoryError(f"{app_name} 目录不存在")


def collect_result(results):
    res = Counter([results.get(i).get("result") for i in results])
    _total = sum(res.values())
    skiped = res.get("skip", 0)
    total = _total - skiped
    passed = res.get("pass", 0)
    failed = total - passed
    pass_rate = f"{round((passed / total) * 100, 2)}%" if passed else "0%"
    return total, failed, passed, skiped, pass_rate


def get_result(ci_result):
    with open(ci_result, "r", encoding="utf-8") as _f:
        results_dict = json.load(_f)
    return collect_result(results_dict)


def write_json(project_name=None, build_location=None, line=None):
    json_tpl_path = f"{GlobalConfig.SETTING_PATH}/template/ci.json"
    if not exists(json_tpl_path):
        raise FileNotFoundError
    with open(json_tpl_path, "r", encoding="utf-8") as _f:
        results = json.load(_f)

    results["project_name"] = project_name
    results["build_location"] = build_location
    results["line"] = line
    ci_result_path = f"{GlobalConfig.ROOT_DIR}/ci_result.json"
    if not exists(ci_result_path):
        return

    (
        results["total"],
        results["fail"],
        results["pass"],
        results["skip"],
        results["pass_rate"],
    ) = get_result(ci_result_path)

    json_res_path = f"{GlobalConfig.ROOT_DIR}/{project_name}_at.json"
    with open(json_res_path, "w+", encoding="utf-8") as _f:
        _f.write(json.dumps(results, indent=2, ensure_ascii=False))
    sleep(1)
    from src import logger
    with open(json_res_path, "r", encoding="utf-8") as _f:
        logger.info("CICD数据结果:\n", _f.read())



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_

# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.

import os
# SPDX-License-Identifier: GPL-2.0-only
# pylint: disable=C0114
import subprocess
import sys

from setting import conf
from src import logger
from src.custom_exception import ShellExecutionFailed


class CmdCtl:
    """命令行工具"""

    # pylint: disable=too-many-arguments,too-many-locals,too-many-public-methods
    # clean_env也要用
    GREP_LIST = (
        "grep",
        "pytest",
        "python",
        "asan",
        "tee",
        "ffmpeg",
        "youqu",
    )

    @staticmethod
    def _run(command, _input=None, timeout=None, check=False, **kwargs):
        """run"""
        with subprocess.Popen(command, **kwargs) as process:
            try:
                stdout, stderr = process.communicate(_input, timeout=timeout)
            except:  # Including KeyboardInterrupt, communicate handled that.
                process.kill()
                raise
            retcode = process.poll()
            if check and retcode:
                raise subprocess.CalledProcessError(
                    retcode, process.args, output=stdout, stderr=stderr
                )
        return subprocess.CompletedProcess(process.args, retcode, stdout, stderr)

    @classmethod
    def _getstatusoutput(cls, command, timeout, executable):
        """getstatusoutput"""
        kwargs = {
            "shell": True,
            "stderr": subprocess.STDOUT,
            "stdout": subprocess.PIPE,
            "timeout": timeout,
            "executable": executable,
        }
        try:
            if sys.version_info >= (3, 7):
                kwargs["text"] = True
            result = cls._run(command, **kwargs
                              )
            data = result.stdout
            if isinstance(data, bytes):
                data = data.decode("utf-8")
            exitcode = result.returncode
        except subprocess.CalledProcessError as ex:
            data = ex.output
            exitcode = ex.returncode
        except subprocess.TimeoutExpired as ex:
            # pylint: disable=unnecessary-dunder-call
            data = ex.__str__()
            exitcode = -1
        if data[-1:] == "\n":
            data = data[:-1]
        return exitcode, data

    @classmethod
    def sudo_run_cmd(
            cls,
            command,
            workdir: str = None,
            interrupt: bool = False,
            timeout: int = 25,
            out_debug_flag: bool = True,
            command_log: bool = True,
            password: str = None,
    ):
        if password is None:
            password = conf.PASSWORD
        wd = ""
        if workdir:
            if not os.path.exists(workdir):
                raise FileNotFoundError
            wd = f"cd {workdir} && "
        return cls.run_cmd(
            f"{wd}echo '{password}' | sudo -S {command}",
            interrupt=interrupt,
            timeout=timeout,
            out_debug_flag=out_debug_flag,
            command_log=command_log,
        )

    @classmethod
    def run_cmd(
            cls,
            command,
            interrupt=False,
            timeout=25,
            out_debug_flag=True,
            command_log=True,
            executable="/bin/bash"
    ):
        """
         执行shell命令
        :param command: shell 命令
        :param interrupt: 命令异常时是否中断
        :param timeout: 命令执行超时
        :param out_debug_flag: 命令返回信息输出日志
        :param command_log: 执行的命令字符串日志
        :return: 返回终端输出
        """
        status, out = cls._getstatusoutput(command, timeout=timeout, executable=executable)
        if command_log:
            logger.debug(command)
        if status and interrupt:
            raise ShellExecutionFailed(out)
        if out_debug_flag and out:
            logger.debug(out)
        return out

    @classmethod
    def monitor_process(cls, app_name, grep_list: str = None):
        """
         监控进程状态
        :param app_name: 应用包名
        :return: 进程 ID，进程名，
        """
        if grep_list:
            cls.GREP_LIST = grep_list
        cmd = ""
        for i in cls.GREP_LIST:
            cmd += f"grep -v {i} | "
        ps_grep_v_str = (
            f"ps -aux | grep {app_name} | {cmd.rstrip('| ')} | "
            # pylint: disable=anomalous-backslash-in-string
            "grep -v daemon | grep -v '\-d'"
        )

        p_id = cls.run_cmd(
            # pylint: disable=consider-using-f-string
            "%s | awk '{print $2}'" % ps_grep_v_str,
            interrupt=False,
            command_log=False,
        )
        if not p_id:
            return False
        # pylint: disable=consider-using-f-string
        with os.popen("%s | awk '{print $11}'" % ps_grep_v_str) as pro_name:
            p_name = pro_name.readlines()
        # pylint: disable=consider-using-f-string
        with os.popen("%s | awk '{print $12}'" % ps_grep_v_str) as pro_name_suffix:
            p_name_suffix = pro_name_suffix.readlines()

        if not p_name_suffix:
            logger.info(f"{app_name} start failed")

        logger.info(f"{app_name} | 进程查找结果为:{p_id}, {p_name}, {p_name_suffix}")
        return p_id, p_name, p_name_suffix

    @classmethod
    def get_process_status(cls, app: str, grep_list: str = None) -> bool:
        """
         获取进程状态
        :param app: 应用包名
        :return: Boolean
        """
        if grep_list:
            cls.GREP_LIST = grep_list
        cmd = ""
        for i in cls.GREP_LIST:
            cmd += f"grep -v {i} | "
        cmd_txt = f"ps -aux | grep {app} | {cmd.rstrip('| ')}"
        logger.debug(cmd_txt)
        command = os.popen(cmd_txt)
        result = command.read()
        command.close()
        if result:
            logger.debug(result)
            return True
        return False

    @classmethod
    def get_daemon_process_status(cls, app, grep_list: str = None) -> bool:
        """
         获取有守护进程的应用进程状态
        :param app: 应用包名
        :return: Boolean
        """
        if grep_list:
            cls.GREP_LIST = grep_list
        cmd = ""
        for i in cls.GREP_LIST:
            cmd += f"grep -v {i} | "
        cmd_txt = f"ps -aux | grep -v daemon | grep {app} | {cmd.rstrip('| ')}"
        logger.debug(cmd_txt)
        command = os.popen(cmd_txt)
        result = command.read()
        logger.debug(result)
        command.close()
        return bool(result)

    @classmethod
    def get_daemon_process_num(cls, app, grep_list: str = None) -> int:
        """
         获取有守护进程的应用进程数量
        :param app:
        :return: int
        """
        if grep_list:
            cls.GREP_LIST = grep_list
        cmd = ""
        for i in cls.GREP_LIST:
            cmd += f"grep -v {i} | "
        cmd_txt = f"ps -aux | grep -v daemon | grep {app} | {cmd.rstrip('| ')}"
        logger.debug(cmd_txt)
        command = os.popen(cmd_txt)
        result = command.read()
        command.close()
        logger.debug(result)
        num = len(result.split("\n")) - 1
        return num

    @staticmethod
    def minimize_current_window():
        """
         最小化当前激活窗口
        :return: None
        """
        with os.popen("xdotool windowminimize $(xdotool getactivewindow)"):
            pass

    @classmethod
    def change_sys_icon_theme(cls, theme="bloom"):
        """
         修改系统的图标主题
        :param theme: 默认 bloom. 支持：Vintage,bloom,bloom-classic,bloom-classic-dark,bloom-dark
        """
        logger.debug(f"设置系统图标主题为{theme}")
        cls.run_cmd(f'gsettings set com.deepin.xsettings icon-theme-name "{theme}"')

    @classmethod
    def change_app_to_default_theme(cls, app_name):
        """
         修改app主题跟随系统
        :param app_name: 应用名字
        """
        cls.run_cmd(
            f"gsettings set com.deepin.dtk:/dtk/deepin/{app_name}/ " "palette-type UnknownType"
        )

    @classmethod
    def kill_process(cls, process, grep_list: [list, tuple] = None):
        """
         杀进程
        :param process: 进程名
        """
        # 杀进程要过滤的进程
        if grep_list:
            cls.GREP_LIST = grep_list
        cmd = ""
        for i in cls.GREP_LIST:
            cmd += f"grep -v {i} | "
        os.system(f"ps -ef | grep {process} | {cmd}awk '{{print $2}}' | xargs kill -9 > /dev/null 2>&1")

    @classmethod
    def sudo_kill_process(cls, process, grep_list: [list, tuple] = None):
        if grep_list:
            cls.GREP_LIST = grep_list
        cmd = ""
        for i in cls.GREP_LIST:
            cmd += f"grep -v {i} | "
        from setting import conf
        os.system(
            f"process=$(ps -ef | grep {process} | {cmd}cut -c 9-15);echo '{conf.PASSWORD}' | sudo -S kill -9 $process > /dev/null 2>&1"
        )

    @staticmethod
    def expect_run(
            cmd: str,
            events: dict,
            return_code=False,
            timeout: int = 30
    ):
        """
        expect_run(
            "ssh username@machine_ip 'ls -l'",
            events={'password':'secret\n'}
        )
        如果 return_code=True，返回 (stdout, return_code)
        """
        import pexpect
        return pexpect.run(
            cmd,
            events=events,
            withexitstatus=return_code,
            timeout=timeout
        )



from enum import Enum
from enum import unique
from setting.globalconfig import GlobalConfig


def help_tip():
    color = "32"
    return (
        f'''\033[1;{color}mYouQu\033[0m 支持的子命令:\n    '''
        f'''\033[1;{color}m{str([i.value for i in SubCmd]).replace("'", "").strip("[").strip("]")}\033[0m'''
        f"\n您需要传入其中一个子命令,可以使用 \033[1;{color}m-h\033[0m 或 \033[1;{color}m--help\033[0m 查看每个命令参数的详细使用说明."
        f"\n如: \033[1;{color}myouqu manage.py run -h\033[0m \n"
    )


def start_app(startapp=None):
    if startapp:
        from src.startapp import StartApp

        start = StartApp(startapp)
        start.copy_template_to_apps()
        start.rewrite()


def trim():
    from src.depends.cfonts import say

    say(GlobalConfig.PROJECT_NAME)
    version_font = "slick"
    color = "32"
    say(GlobalConfig.current_tag, font=version_font, space=False)
    say(f"Code: \033[1;{color}m{GlobalConfig.GITHUB_URL}\033[0m", font="console", space=False)
    say(f"Docs: \033[1;{color}m{GlobalConfig.DOCS_URL}\033[0m", font="console", space=False)
    say(f"PyPI: \033[1;{color}m{GlobalConfig.PyPI_URL}\033[0m", font="console", space=False)
    say(f'\033[1;{color}m{"=" * 60}\033[0m', font="console", space=False)


@unique
class SubCmd(Enum):
    run = "run"
    remote = "remote"
    pmsctl = "pmsctl"
    csvctl = "csvctl"
    startapp = "startapp"
    git = "git"



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
from typing import Union, List

from funnylog import logger

from setting import conf
from src import Src
from src.cmdctl import CmdCtl
from src.dogtail_utils import DogtailUtils
from src.remotectl._remote_dogtail_ctl import remote_dogtail_ctl as remote_dogtail_ctl
from src.remotectl._remote_other_ctl import remote_other_ctl as remote_other_ctl
from src.shortcut import ShortCut


class Remote(ShortCut, CmdCtl):
    def __init__(self, ip, user, password, transfer_appname=None, restart_service=False):
        self.user = user
        self.ip = ip
        self.password = password
        self.transfer_appname = transfer_appname
        self.restart_service = restart_service
        self.tmp_obj = None

    def __getattribute__(self, item):
        if not item.startswith("__") and not item.endswith("__"):
            for cls_obj in [ShortCut, CmdCtl]:
                if hasattr(cls_obj, item):
                    self.tmp_obj = {"cls_obj": cls_obj, "item_obj": getattr(cls_obj, item)}
                    while True:
                        if hasattr(cls_obj, item):
                            delattr(cls_obj, item)
                        else:
                            break
        return super().__getattribute__(item)

    def __getattr__(self, item):
        def func(*args, **kwargs):
            ar = ""
            if args:
                for arg in args:
                    ar += f"'{arg}', "
            if kwargs:
                for k, v in kwargs.items():
                    ar += f"{k}='{v}', "
            logger.info(
                f"Remote(user='{self.user}', ip='{self.ip}', password='{self.password}').rctl.{item}({ar.rstrip(', ')})"
            )
            value = None
            try:
                value = getattr(self.rctl, item)(*args, **kwargs)
            finally:
                if self.tmp_obj:
                    setattr(self.tmp_obj["cls_obj"], item, self.tmp_obj["item_obj"])
                    self.tmp_obj = None
            return value

        return func

    @property
    def rdog(self) -> DogtailUtils:
        return remote_dogtail_ctl(
            user=self.user,
            ip=self.ip,
            password=self.password,
            restart_service=self.restart_service,
        )

    def click_element_by_attr(self, element, button=1):
        self.rdog.element_click(element, button)

    def get_element_children_txt(self, element):
        return self.rdog.get_element_children_text(element)

    def get_element_center(self, element):
        return self.rdog.element_center(element)

    @property
    def rctl(self) -> Src:
        return remote_other_ctl(
            user=self.user,
            ip=self.ip,
            password=self.password,
            restart_service=self.restart_service,
        )

    @property
    def rctl_plus(self) -> Src:
        return remote_other_ctl(
            user=self.user,
            ip=self.ip,
            password=self.password,
            transfer_appname=self.transfer_appname,
            restart_service=self.restart_service,
        )

    def find_image(
            self,
            *image,
            rate: Union[float, int] = None,
            multiple: bool = False,
            picture_abspath: str = None,
            screen_bbox: List[int] = None,
            log_level: str = "info",
            network_retry: int = None,
            pause: [int, float] = None,
            timeout: [int, float] = None,
            max_match_number: int = None
    ):
        """
         在屏幕中区寻找小图，返回坐标，
         如果找不到，根据配置重试次数，每次间隔1秒
        :param picture_abspath: 原始图片绝对路径，不指定时截全屏
        :param image: 模板图片路径
        :param rate: 相似度
        :param multiple: 是否返回匹配到的多个目标
        :param screen_bbox: 截取屏幕上指定区域图片（仅支持X11下使用）；
            [x, y, w, h]
            x: 左上角横坐标；y: 左上角纵坐标；w: 宽度；h: 高度；根据匹配度返回坐标
        :param log_level: 日志级别
        :param network_retry: 连接服务器重试次数
        :param pause: 图像识别重试的间隔时间
        :param timeout: 最大匹配超时,单位秒
        :param max_match_number: 最大匹配次数
        :return: 坐标元组
        """
        _image_path = tuple([_path.replace(conf.HOME, "~", 1) for _path in image])
        if picture_abspath:
            picture_abspath = picture_abspath.replace(conf.HOME, "~", 1)
        return self.rctl_plus.find_image_remote(
            _image_path, rate, multiple, picture_abspath, screen_bbox, log_level,
            network_retry, pause, timeout, max_match_number
        )

    def ocr(
            self,
            *target,
            picture_abspath: str = None,
            similarity: [int, float] = 0.6,
            return_default: bool = False,
            return_first: bool = False,
            lang: str = "ch",
            network_retry: int = None,
            pause: [int, float] = None,
            timeout: [int, float] = None,
            max_match_number: int = None
    ):
        """
        通过 OCR 进行识别。
        :param target:
            目标字符,识别一个字符串或多个字符串,并返回其在图片中的坐标;
            如果不传参，返回图片中识别到的所有字符串。
        :param picture_abspath: 要识别的图片路径，如果不传默认截取全屏识别。
        :param similarity: 匹配度。
        :param return_default: 返回识别的原生数据。
        :param return_first: 只返回第一个,默认为 False,返回识别到的所有数据。
        :param lang: `ch`, `en`, `fr`, `german`, `korean`, `japan`
        :param network_retry: 连接服务器重试次数
        :param pause: 重试间隔时间,单位秒
        :param timeout: 最大匹配超时,单位秒
        :param max_match_number: 最大匹配次数
        :return: 返回的坐标是目标字符串所在行的中心坐标。
        """
        if picture_abspath:
            picture_abspath = picture_abspath.replace(conf.HOME, "~", 1)
        return self.rctl_plus.ocr_remote(target, picture_abspath, similarity, return_default, return_first, lang,
                                         network_retry,
                                         pause, timeout, max_match_number)

if __name__ == '__main__':
    a = Remote(
        user="uos",
        ip="10.8.7.55",
        password="1",
    ).rcmd.sudo_run_cmd("systemctl restart lightdm.service")
    # ).run_cmd("ls")
    print(a)
